import pathlib
import unittest

from voc_tools import reader as voc_reader
from voc_tools.utils import Annotation, VOCDataset
from voc_tools.visulizer import from_jpeg, see_jpeg


class MyTestCase(unittest.TestCase):
    def test_something(self):
        dataset_path = pathlib.Path(r"sixray_data")

        self.assertEqual(list(voc_reader.list_dir(str(dataset_path / "train"), fullpath=False)),
                         ["P00002.xml", "P00003.xml", "P00004.xml"], msg="XML files not matched.")
        self.assertEqual(list(voc_reader.list_dir(str(dataset_path / "train"), fullpath=False, images=True)),
                         ["P00002.jpg", "P00003.jpg", "P00004.jpg"], msg="JPEG files not matched.")
        self.assertEqual(Annotation.csv_header(), "file,xmin,ymin,xmax,ymax,center_x,center_y,class_name",
                         msg="CSV header mismatch")

        ["file:P00002.jpg,xmin:274,ymin:232,xmax:610,ymax:460,center_x:442.0,center_y:346.0,class_name:knife",
         "file:P00002.jpg,xmin:225,ymin:334,xmax:591,ymax:465,center_x:408.0,center_y:399.5,class_name:knife", ]
        ["P00002.jpg,274,232,610,460,442.0,346.0,knife",
         "P00002.jpg,225,334,591,465,408.0,399.5,knife",
         "P00002.jpg,274,232,610,460,442.0,346.0,knife",
         "P00002.jpg,225,334,591,465,408.0,399.5,knife", ]
        "P00003.jpg,858,121,918,321,888.0,221.0,knife",
        "P00004.jpg,83,73,133,425,108.0,249.0,knife",
        "P00004.jpg,1,162,343,318,172.0,240.0,knife",

        for anno in voc_reader.from_file(r"sixray_data\train\JPEGImages\P00002.jpeg"):
            print(str(anno))
        for anno in voc_reader.from_file(r"sixray_data\train\Annotations\P00002.xml"):
            print(anno.csv())
        for anno in voc_reader.from_dir(str(dataset_path / "train")):
            print(anno.csv())
        self.assertEqual("file,xmin,ymin,xmax,ymax,center_x,center_y,class_name", Annotation.csv_header())

        my_voc = VOCDataset(str(dataset_path))
        my_voc.train.load()
        my_voc.train.unload()
        my_voc.train.to_csv(str(dataset_path / "train.csv"))
        classes1 = my_voc.train.class_names()
        classes2 = "knife",
        self.assertEqual(classes2, classes1)

        voc_caption_data = VOCDataset(str(dataset_path), caption_support=True)
        for caption in voc_caption_data.train.caption.fetch():
            print(caption)

        for anno, jpeg in voc_caption_data.train.fetch():
            print(anno, jpeg)

        voc_caption_data.train.caption.to_csv(str(dataset_path / "captions.csv"))
        jpg = from_jpeg(r"sixray_data\train\JPEGImages\P00002.jpg")
        shape1 = jpg.image.shape
        shape2 = (482, 801, 3)
        self.assertEqual(shape1, shape2)
        # see_jpeg(r"sixray_data\train\JPEGImages\P00002.jpg")


if __name__ == '__main__':
    unittest.main()



import os
import pathlib
from abc import ABC, abstractmethod

import cv2
import numpy as np

from voc_tools.constants import VOC_IMAGES, VOC_CAPTIONS, VOC_XMLS


def path_convert(file: str, converter_flag: int):
    """
    Convert any file path to a required file path
    :param file: A path to xml, jpeg or txt file
    :param converter_flag:
    :return: Converted path
    """
    assert file.endswith(".xml") or file.endswith(".txt") or file.endswith(".jpeg") or file.endswith(".jpg"), (
        "Unsupported file format")
    assert converter_flag in [VOC_IMAGES, VOC_CAPTIONS, VOC_XMLS], "Unsupported converter_flag"
    file = pathlib.Path(file)
    filename = file.name
    parent_path = file.parents[1]
    if converter_flag == VOC_IMAGES:
        path = parent_path / "JPEGImages" / filename
    if converter_flag == VOC_XMLS:
        path = parent_path / "Annotations" / filename
    if converter_flag == VOC_CAPTIONS:
        path = parent_path / "text" / filename
    return str(path)


# ################################################
# Python classes to hold annotation and captions #
# ################################################
class Atomic:
    def __str__(self):
        s = ""
        for k, v in zip(self.raw_attributes(), self.raw()):
            s = "{}{}:{},".format(s, k, v)
        return s.strip(",")

    @classmethod
    def csv_header(cls):
        """Return the CSV header. A method instance for a single class is enough"""
        return ",".join(cls.raw_attributes())

    def csv(self):
        """Returns the CSV formatted data as string"""
        return ",".join(["{}"] * len(self.raw())).format(*self.raw())

    @abstractmethod
    def raw(self):
        """Return raw data as tuple"""
        ...

    @classmethod
    @abstractmethod
    def raw_attributes(cls):
        """Return raw tuple of strings(to be used as key per values). A single method instance is required per class"""
        ...


class JPEG(Atomic):

    def raw(self):
        return self._filename, *self._cv_image.shape

    @classmethod
    def raw_attributes(cls):
        return "file", "height", "width", "channels"

    def __init__(self, filename):
        filename = path_convert(filename, VOC_IMAGES)
        self._filename = pathlib.Path(filename).name
        self._cv_image = cv2.imread(filename)

    @property
    def filename(self):
        return self._filename

    @property
    def image(self):
        return self._cv_image

    def see(self):
        cv2.imshow(self.filename, self._cv_image)
        cv2.waitKey(-1)


class Annotation(Atomic):
    def __init__(self, filename, xmin, ymin, xmax, ymax, center_x, center_y, class_name):
        self._xmin, self._ymin, self._xmax, self._ymax, self._center_x, self._center_y, self._class_name = (
            xmin, ymin, xmax,
            ymax, center_x,
            center_y,
            class_name)
        self._filename = filename

    @property
    def filename(self):
        return self._filename

    @property
    def xmin(self):
        return self._xmin

    @property
    def ymin(self):
        return self._ymin

    @property
    def xmax(self):
        return self._xmax

    @property
    def ymax(self):
        return self._ymax

    @property
    def center_x(self):
        return self._center_x

    @property
    def center_y(self):
        return self._center_y

    @property
    def class_name(self):
        return self._class_name

    @classmethod
    def raw_attributes(cls):
        return "file", "xmin", "ymin", "xmax", "ymax", "center_x", "center_y", "class_name"

    def raw(self):
        return (self._filename, self._xmin, self._ymin, self._xmax, self._ymax, self._center_x, self._center_y,
                self._class_name)


class Caption(Atomic):
    def __init__(self, filename, caption):
        self._filename = filename
        self._caption = caption

    @property
    def filename(self):
        return self._filename

    @property
    def captions(self):
        return self._caption

    @classmethod
    def raw_attributes(cls):
        return "file", "caption"

    def raw(self):
        return self._filename, self._caption


# ########################################################
# Python classes to hold annotation and captions dataset #
# ########################################################
class ABCDataset(ABC):
    def __init__(self, dataset_path):
        assert os.path.exists(dataset_path), dataset_path
        self.dataset_path = dataset_path

    @abstractmethod
    def to_csv(self, path_to_csv, write_mode="w"):
        ...


class CaptionDataset(ABCDataset):
    def __init__(self, dataset_path):
        super().__init__(dataset_path)

    def fetch(self):
        """
        Generate Caption object
        """
        from voc_tools.reader import caption_from_dir

        for caption in caption_from_dir(self.dataset_path):
            yield caption

    def to_csv(self, path_to_csv, write_mode="w"):
        """
        Generate csv file for given VOC dataset
        """
        from voc_tools.reader import caption_from_dir
        with open(path_to_csv, write_mode) as csv_fp:
            csv_fp.write("{}\n".format(Caption.csv_header()))
            for caption in caption_from_dir(self.dataset_path):
                csv_fp.write("{}\n".format(caption.csv()))
        return self


class Dataset(ABCDataset):
    def __init__(self, dataset_path, caption_support=False):
        super().__init__(dataset_path)
        self.dataset_path = dataset_path
        self.caption_support = caption_support
        self.meta = np.array([], dtype='object')
        self.image_cache: JPEG = None
        self.loaded = False
        if caption_support:
            self.caption = CaptionDataset(dataset_path)

    def load(self):
        if not self.loaded:
            from voc_tools.reader import from_dir
            self.meta = np.array([anno.raw() for anno in from_dir(self.dataset_path)], dtype='object')
            self.loaded = True
        return self

    def unload(self):
        del self.meta
        self.meta = np.array([], dtype='object')
        self.loaded = False
        return self

    def fetch(self):
        from voc_tools.reader import from_dir
        for anno in from_dir(self.dataset_path):
            yield anno, self.get_image(anno.filename)

    def class_names(self):
        if not self.loaded:
            self.load()
        class_name_idx = Annotation.raw_attributes().index('class_name')
        return tuple(set(self.meta[:, class_name_idx]))

    def get_image(self, filename):
        return JPEG(os.path.join(self.dataset_path, "JPEGImages", filename))

    def get_image_meta(self, filename):
        if self.image_cache is None or filename != self.image_cache.filename:
            # save the image
            self.image_cache = self.get_image(filename)
            # if a newer appear, re
        return self.image_cache.csv().split(",")[1:]

    def to_csv(self, path_to_csv, write_mode="w"):
        """
        Generate csv file for given VOC dataset
        """
        from voc_tools.reader import from_dir
        with open(path_to_csv, write_mode) as csv_fp:
            csv_fp.write("{}".format(Annotation.csv_header()))
            csv_fp.write("{}\n".format(JPEG.csv_header().replace("file", "")))
            for anno in from_dir(self.dataset_path):
                csv_fp.write("{},".format(anno.csv()))
                csv_fp.write(",".join(self.get_image_meta(anno.filename)))
                csv_fp.write("\n")
        return self


class TrainDataset(Dataset):
    def __init__(self, dataset_path, caption_support=False):
        super().__init__(dataset_path, caption_support=caption_support)


class TestDataset(Dataset):
    def __init__(self, dataset_path, caption_support=False):
        super().__init__(dataset_path, caption_support=caption_support)


# ####################################
# Python classes to hold VOC Dataset #
# ####################################

class VOCDataset:
    def __init__(self, dataset_path, caption_support=False):
        self.dataset_path = dataset_path
        self.caption_support = caption_support
        self.train = TrainDataset(os.path.join(self.dataset_path, "train"), caption_support=caption_support)
        self.test = TestDataset(os.path.join(self.dataset_path, "test"), caption_support=caption_support)



import os
import pathlib
import xml.etree.ElementTree as ET

from voc_tools.constants import VOC_XMLS, VOC_IMAGES, VOC_CAPTIONS
from voc_tools.utils import Annotation, Caption


def from_file(file: str):
    """
    Generate a list of Annotation objects for a given image or xml of a PASCAL VOC dataset.
    It also supports captions .txt files
    """
    if file.endswith(".xml"):
        return from_xml(file)
    elif file.endswith(".txt"):
        return caption_from_file(file)
    elif file.endswith(".jpeg") or file.endswith(".jpg"):
        return from_image(file)
    else:
        raise ValueError("Unsupported file format")


def from_image(image_file: str):
    """
    Generate a list of Annotation objects for a given image of a PASCAL VOC dataset
    """
    image_file = pathlib.Path(image_file)
    parent_path = image_file.parents[1] / "Annotations"
    file_name = image_file.name.replace(".jpeg", ".xml").replace(".jpg", ".xml")
    xml_file = str(parent_path / file_name)
    return from_xml(xml_file)


def from_xml(xml_file: str, empty_placeholder="NULL"):
    """
    Generate a list of Annotation objects from a given VOC XML file
    """
    filename = pathlib.Path(xml_file).name
    try:
        tree = ET.parse(xml_file)
        root = tree.getroot()

        no_threat = True

        filename = root.find('filename').text
        for boxes in root.iter('object'):
            no_threat = False
            class_ = boxes.find("name").text
            ymin = int(boxes.find("bndbox/ymin").text)
            xmin = int(boxes.find("bndbox/xmin").text)
            ymax = int(boxes.find("bndbox/ymax").text)
            xmax = int(boxes.find("bndbox/xmax").text)
            cx = (xmin + xmax) / 2
            cy = (ymin + ymax) / 2

            single_annotation = Annotation(filename, xmin, ymin, xmax, ymax, cx, cy, class_)
            yield single_annotation
        if no_threat:
            yield Annotation(filename, 0, 0, 0, 0, 0, 0, empty_placeholder)
    except Exception as e:
        yield Annotation(filename, 0, 0, 0, 0, 0, 0, "ERROR:{}".format(e))


def list_dir(dir_path: str, dir_flag=VOC_XMLS, images=False, fullpath=True):
    """
    Generate a list of XML files form a given PASCAL VOC directory
    Args:
        dir_path: A path to VOC dataset
        dir_flag: A flag which specify the subdirectory to list. Any typical
                    Pascal VOC dataset must contain Annotations, JPEGImages and text[optionsl] directories.
                    VOC_IMAGES - JPEGImages
                    VOC_XMLS - Annotations
                    VOC_CAPTIONS - text
        images: [depreciated] use this flag to load JPEGImages directory. Use dir_flag=VOC_IMAGES instead.
        fullpath: A boolean flag indicates whether to generate full path or the filename.
    """
    dir_path = pathlib.Path(dir_path)
    if images:
        dir_flag = VOC_IMAGES
    annotations_dir = dir_path / (
        "JPEGImages" if dir_flag == VOC_IMAGES else ("Annotations" if dir_flag == VOC_XMLS else "text"))
    for file_item in os.listdir(str(annotations_dir)):
        if fullpath:
            yield str(annotations_dir / file_item)
        else:
            yield (annotations_dir / file_item).name


def from_dir(dir_path: str):
    """
    Generate a list of Annotation object per file form a given PASCAL VOC directory
    """
    for xml_file in list_dir(dir_path):
        for annotation in from_xml(xml_file):
            yield annotation


def caption_from_file(file_path: str, empty_placeholder="NULL"):
    """
    Generate a list of captions object per file form a given text file path directory
    """
    filename = pathlib.Path(file_path).name
    try:
        with open(file_path, "r") as fp:
            no_caption = True
            for caption in map(lambda x: Caption(filename, x.strip()),
                               filter(lambda l: l.strip() != "", fp.readlines())):
                no_caption = False
                yield caption
            if no_caption:
                yield Caption(filename, empty_placeholder)
    except Exception as e:
        yield Caption(filename, "ERROR:{}".format(e))


def caption_from_dir(dir_path: str):
    """
    Generate a list of captions object per file form a given directory
    """
    for file in list_dir(dir_path, dir_flag=VOC_CAPTIONS):
        for caption in caption_from_file(file):
            yield caption



import pathlib
from setuptools import find_packages, setup

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name='express-pascal-voc-tools',
    description="A tool for creating, reading and visualizing Pascal VOC annotations",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Soumen Sardar",
    author_email="soumensardarintmain@gmail.com",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['lxml', 'numpy', 'opencv-python'],
    version_config=True,
    version="0.5.1"
    # setup_requires=["setuptools-git-versioning"]
)
